from datetime import datetime
from ..database import db
from flask import current_app
from werkzeug.local import LocalProxy
from secrets import compare_digest

logger = LocalProxy(lambda: current_app.logger)


class AccessToken(db.Model):
    """ Model for an AccessToken

        Attributes:
            timestamp: Is a Datetime from current Time.
            user:   Is an User.
            token:  String to verify access later.
    """
    __tablename__ = 'session'
    id = db.Column(db.Integer, primary_key=True)
    timestamp = db.Column(db.DateTime, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    user = db.relationship("User", back_populates="sessions")
    token = db.Column(db.String(30))
    lifetime = db.Column(db.Integer)
    browser = db.Column(db.String(30))
    platform = db.Column(db.String(30))

    def updateTimestamp(self):
        """ Update the Timestamp

            Update the Timestamp to the current Time.
        """
        logger.debug("update timestamp from accesstoken {{ {} }}".format(self))
        self.timestamp = datetime.utcnow()

    def toJSON(self):
        """ Create Dic to dump in JSON

            Returns:
                A Dic with static Attributes.
        """
        dic = {
            "id": self.id,
            "timestamp": {'year': self.timestamp.year,
                          'month': self.timestamp.month,
                          'day': self.timestamp.day,
                          'hour': self.timestamp.hour,
                          'minute': self.timestamp.minute,
                          'second': self.timestamp.second
                          },
            "lifetime": self.lifetime,
            "browser": self.browser,
            "platform": self.platform
        }
        return dic

    def __eq__(self, token):
        return compare_digest(self.token, token)

    def __sub__(self, other):
        return other - self.timestamp

    def __str__(self):
        return "AccessToken(user={}, token={}, timestamp={}, lifetime={}".format(self.user, self.token, self.timestamp, self.lifetime)

    def __repr__(self):
        return "AccessToken(user={}, token={}, timestamp={}, lifetime={}".format(self.user, self.token, self.timestamp, self.lifetime)
